/**
 * Created by janis dähne.
 */

import * as React from "react";

import LoginForm from "./components/login/loginForm";
import {connect, Provider} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {setUsername} from "./state/actions/loginActions";
import {returntypeof} from "react-redux-typescript";
import SiteHeaderBar from './components/siteHeaderBar/siteHeaderBar'
import {RootState} from "./state/reducers";
import {initial as userDataInitial} from './state/reducers/settingsSite/settingsSiteReducers'
import HomeSite from './components/sites/homeSite/homeSite'

import PageContentWrapper from './components/pageContentWrapper'

import SystemRolesSite from './components/sites/systemRolesSite/systemRolesSite'
import SystemRolesHeaderBarContent from './components/sites/systemRolesSite/headerBarContent'

import ManageActivatedUsersSite from './components/sites/manageActivatedUsersSite/manageActivatedUsersSite'
import ManageActivatedUsersHeaderBarContent from './components/sites/manageActivatedUsersSite/headerBarContent'

import ManageNewUsersSite from './components/sites/manageNewUsersSite/manageNewUsersSite'
import ManageNewUsersHeaderBarContent from './components/sites/manageNewUsersSite/headerBarContent'

import LoggerWrapper from './components/loggerPanel/LoggerWrapper'

import GroupRoleSite from './components/sites/groupRolesSite/groupRolesSite'
import GroupRolesHeaderBarContent from './components/sites/groupRolesSite/headerBarContent'

import AboutSite from './components/sites/aboutSite/aboutSite'

import SettingSite from './components/sites/ownSettingsSite/ownSettingSite'

import GroupsSite from './components/sites/groupsSite/groupsSite'
import GroupsHeaderBarContent from './components/sites/groupsSite/headerBarContent'

import SingleGroupSite from './components/sites/singleGroupSite/singleGroupSite'
import SingleGroupHeaderBarContent from './components/sites/singleGroupSite/headerBarContent'

import OwnExercisesSite from './components/sites/manageOwnExercisesSite/ownExercisesSite'
import OwnExercisesHeaderBarContent from './components/sites/manageOwnExercisesSite/headerBarContent'

import GroupExercisesSite from './components/sites/manageGroupExercisesSite/groupExercisesSite'
import GroupExercisesHeaderBarContent from './components/sites/manageGroupExercisesSite/headerBarContent'

import ManageTagsSite from './components/sites/manageTagsSite/manageTagsSite'
import ManageTagsHeaderBarContent from './components/sites/manageTagsSite/headerBarContent'

import OpenExercisesSite from './components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite'
import OpenExercisesSiteHeaderBarContent
  from './components/sites/allAccessibleExercisesSite/openExercises/headerBarContent'

import ClosedExercisesSite
  from './components/sites/allAccessibleExercisesSite/closedExercisesSite/allClosedExercisesSite'
import ClosedExercisesSiteHeaderBarContent
  from './components/sites/allAccessibleExercisesSite/closedExercisesSite/headerBarContent'

import ReleaseSite from './components/sites/manageOwnOrGroupExerciseComponents/releasesSite/releasesSite'
import ReleaseSiteHeaderBarContent
  from './components/sites/manageOwnOrGroupExerciseComponents/releasesSite/headerBarContent'

import SubmissionsSite from './components/sites/manageOwnOrGroupExerciseComponents/submissionsSite/submissionsSite'
import SubmissionsSiteHeaderBarContent
  from './components/sites/manageOwnOrGroupExerciseComponents/submissionsSite/headerBarContent'

import OpenExercisesViaCodeSite
  from './components/sites/allAccessibleExercisesSite/openExercises/openExercisesViaCodeSite'
import OpenExercisesViaVisibilitySite
  from './components/sites/allAccessibleExercisesSite/openExercises/openExercisesViaVisibilitySite'

import ClosedExercisesSiteViaCode
  from './components/sites/allAccessibleExercisesSite/closedExercisesSite/closedExercisesSiteViaCode'
import ClosedExercisesSiteViaVisibility
  from './components/sites/allAccessibleExercisesSite/closedExercisesSite/closedExercisesSiteViaVisibility'

import ExerciseEditorSite from './components/sites/exerciseEditorSite/exerciseEditorSite'
import ExerciseEditorSiteHeaderBarContent from './components/sites/exerciseEditorSite/headerBarContent'

import DoExerciseSite from './components/sites/doExerciseSite/doExerciseSite'
import DoExerciseSiteHeaderBarContent from './components/sites/doExerciseSite/headerBarContent'


import {Route, RouteComponentProps, Router, Switch, withRouter} from 'react-router'
import createBrowserHistory from 'history/createBrowserHistory'
import * as constants from "./constants";
import {NotificationContainer} from 'react-notifications'

import ExerciseSyntaxSite from './components/sites/guidesSite/exerciseSyntaxSite'
import TestSyntaxSite from './components/sites/guidesSite/testSyntaxSite'

import TutorViewSite from './components/sites/tutorViewSite/tutorViewSite'
import TutorViewSiteHeaderBarContent from './components/sites/tutorViewSite/headerBarContent'

import AssessmentStatisticsSite from './components/sites/assessmentStatisticSite/assessmentStatisticSite'
import AssessmentStatisticsSiteHeaderBarContent from './components/sites/assessmentStatisticSite/headerBarContent'

import ManageCustomProjectsSite from './components/sites/manageCustomProjectsSite/manageCustomProjectsSite'
import ManageCustomProjectsSiteHeaderBarContent from './components/sites/manageCustomProjectsSite/headerBarContent'

import EditCustomProjectSite from './components/sites/editCustomProjectSite/editCustomProjectSite'
import EditCustomProjectSiteHeaderBarContent from './components/sites/editCustomProjectSite/headerBarContent'

import CommonHeaderBar from './components/commonHeaderBar/commonHeaderBar'

import SiteSideMenuWrapper from './components/siteSideMenu/SiteSideMenuWrapper'

import DashboardSite from './components/sites/dashboardSite/dashboardSite'
import DashboardSiteHeaderBarContent from './components/sites/dashboardSite/headerBarContent'
import Spinner from './components/helpers/spinner'

import SystemSettingsSite from './components/sites/systemSettingsSite/systemSettingsSite'

import {tempPrintDivId} from './constants'
import Chat404 from './components/404internal/Chat404'

const mapStateToProps = (rootState: RootState) => {
  return {
    userId: rootState.userDataSettingsSate.userData.id,
    isSiteHeaderBarDisplayed: rootState.siteHeaderBarState.isSiteHeaderBarDisplayedAndEnabled,
    isSiteHeaderBarCollapsed: rootState.siteHeaderBarState.isSiteHeaderBarCollapsed,
    isAddUsersDialogDisplayed: rootState.singleGroupState.isAddUsersDialogDisplayed,
    isLogginOut: rootState.siteHeaderBarState.isLoggingOut,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        setUsername,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class App extends React.Component<Props, any> {


  render(): JSX.Element {
    return (
      <div className="fh">

        {
          this.props.isLogginOut &&
          <Spinner/>
        }

        <SiteHeaderBar/>


        <SiteSideMenuWrapper/>


        <div
          className={this.props.isSiteHeaderBarDisplayed && !this.props.isSiteHeaderBarCollapsed
                     ? 'fh site-content'
                     : 'fh site-content-no-site-header-bar'}>
          <div className="fh relative-positioned">
            {
              //TODO i think this is not needed anymore because the login overlay handles this already??
              this.props.userId === userDataInitial.userData.id &&
              <LoginForm/>
            }

            <Switch>
              {
                //syntax guides
              }
              <Route exact path={constants.markdownSiteExerciseSyntaxGuide} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper>
                  <ExerciseSyntaxSite/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.markdownSiteTestSyntaxGuide} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper>
                  <TestSyntaxSite/>
                </PageContentWrapper>
              }/>


              {
                //start site
              }
              <Route exact path={constants.homeLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper>
                  <HomeSite/>
                </PageContentWrapper>
              }/>

              {
                //dashboard

                <Route exact path={constants.dashboardLinkPath} render={(props: RouteComponentProps<any>) =>
                  <PageContentWrapper
                    //contains nothing yet
                    // headerBar={
                    //   <CommonHeaderBar>
                    //     <DashboardSiteHeaderBarContent/>
                    //   </CommonHeaderBar>
                    // }
                  >
                    <DashboardSite/>
                  </PageContentWrapper>
                }/>
              }

              {
                //manage custom projects site
              }
              <Route exact path={constants.manageCustomProjectsSiteLinkMatchPath}
                     render={(props: RouteComponentProps<any>) =>
                       <PageContentWrapper
                         headerBar={
                           <CommonHeaderBar>
                             <ManageCustomProjectsSiteHeaderBarContent/>
                           </CommonHeaderBar>
                         }
                       >
                         <ManageCustomProjectsSite/>
                       </PageContentWrapper>
                     }/>

              {
                //edit custom projects site (create, edit)
              }
              <Route exact path={constants.createCustomProjectSiteToLinkPath}
                     render={(props: RouteComponentProps<any>) =>
                       <PageContentWrapper
                         headerBar={
                           <CommonHeaderBar>
                             <EditCustomProjectSiteHeaderBarContent/>
                           </CommonHeaderBar>
                         }
                       >
                         <EditCustomProjectSite {...props}/>
                       </PageContentWrapper>
                     }/>

              <Route exact path={constants.getCustomProjectMatchLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <EditCustomProjectSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <EditCustomProjectSite {...props}/>
                </PageContentWrapper>
              }/>


              {
                //assessment statistics site
              }
              <Route exact path={constants.assessmentStatisticsSiteLinkMatchPath}
                     render={(props: RouteComponentProps<any>) =>
                       <PageContentWrapper
                       >
                         <AssessmentStatisticsSite {...props}/>
                       </PageContentWrapper>
                     }/>


              {
                //tutor view
              }
              <Route exact path={constants.tutorViewSiteLinkMatchPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <TutorViewSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <TutorViewSite {...props}/>
                </PageContentWrapper>
              }/>

              {
                //exercise editor create, update, copy
              }
              <Route exact path={constants.exerciseEditorCreateNewLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ExerciseEditorSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ExerciseEditorSite {...props}/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.exerciseEditorChangeLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ExerciseEditorSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ExerciseEditorSite {...props}/>
                </PageContentWrapper>
              }/>

              {
                //do exercise site
              }
              <Route exact path={constants.doExerciseSiteLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <DoExerciseSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <DoExerciseSite {...props}/>
                </PageContentWrapper>
              }/>


              {
                // --- manage exercises ---
              }
              <Route exact path={constants.ownExercisesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <OwnExercisesHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <OwnExercisesSite/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.groupExercisesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <GroupExercisesHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <GroupExercisesSite/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.exerciseReleasesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ReleaseSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ReleaseSite {...props} />
                </PageContentWrapper>
              }/>

              <Route exact path={constants.submissionsSiteLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <SubmissionsSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <SubmissionsSite {...props}/>
                </PageContentWrapper>
              }/>

              {
                //do exercises sites

              }

              <Route exact path={constants.openExercisesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <OpenExercisesSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <OpenExercisesSite/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.closedExercisesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ClosedExercisesSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ClosedExercisesSite/>
                </PageContentWrapper>
              }/>

              {
                //open exercises sites
              }
              <Route exact path={constants.openExercisesViaCodeLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <OpenExercisesSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <OpenExercisesViaCodeSite shouldLoadData={true} canDisplayEnterReleaseCodeDialog={true}
                                            canDisplayTagFilterPanel={true}/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.openExercisesViaVisibilityLinkPath}
                     render={(props: RouteComponentProps<any>) =>
                       <PageContentWrapper
                         headerBar={
                           <CommonHeaderBar>
                             <OpenExercisesSiteHeaderBarContent/>
                           </CommonHeaderBar>
                         }
                       >
                         <OpenExercisesViaVisibilitySite shouldLoadData={true} canDisplayEnterReleaseCodeDialog={true}
                                                         canDisplayTagFilterPanel={true}/>
                       </PageContentWrapper>
                     }/>

              {
                //closed exercises sites
              }
              <Route exact path={constants.closedExercisesViaCodeLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ClosedExercisesSiteHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ClosedExercisesSiteViaCode shouldLoadData={true} canDisplayTagFilterPanel={true}/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.closedExercisesViaVisibilityLinkPath}
                     render={(props: RouteComponentProps<any>) =>
                       <PageContentWrapper
                         headerBar={
                           <CommonHeaderBar>
                             <ClosedExercisesSiteHeaderBarContent/>
                           </CommonHeaderBar>
                         }
                       >
                         <ClosedExercisesSiteViaVisibility shouldLoadData={true} canDisplayTagFilterPanel={true}/>
                       </PageContentWrapper>
                     }/>

              {
                // --- groups ---
              }

              <Route exact path={constants.groupsLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <GroupsHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <GroupsSite/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.singleGroupLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <SingleGroupHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <SingleGroupSite {...props}/>
                </PageContentWrapper>
              }/>

              {
                // --- roles ---
              }

              <Route exact path={constants.groupRolesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <GroupRolesHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <GroupRoleSite/>
                </PageContentWrapper>
              }/>

              <Route exact path={constants.systemRolesLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <SystemRolesHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <SystemRolesSite/>
                </PageContentWrapper>
              }/>

              {
                // --- user settings ---
              }
              <Route exact path={constants.ownSettingsLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                >
                  <SettingSite/>
                </PageContentWrapper>
              }/>

              {
                // --- new users ---
              }
              <Route path={constants.manageNewUsersLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ManageNewUsersHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ManageNewUsersSite {...props}/>
                </PageContentWrapper>
              }/>

              {
                // --- activated users ---
              }

              <Route path={constants.manageActivatedUsersLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ManageActivatedUsersHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ManageActivatedUsersSite {...props}/>
                </PageContentWrapper>
              }/>


              {
                // --- tags ---
              }
              <Route exact path={constants.manageTagsLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper
                  headerBar={
                    <CommonHeaderBar>
                      <ManageTagsHeaderBarContent/>
                    </CommonHeaderBar>
                  }
                >
                  <ManageTagsSite/>
                </PageContentWrapper>
              }/>

              {
                //--- system settings
              }
              <Route exact path={constants.systemSettingsSiteLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper>
                  <SystemSettingsSite/>
                </PageContentWrapper>
              }/>

              {
                // --- about ---
              }
              <Route exact path={constants.aboutLinkPath} render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper>
                  <AboutSite/>
                </PageContentWrapper>
              }/>

              <Route render={(props: RouteComponentProps<any>) =>
                <PageContentWrapper>
                  <Chat404/>
                </PageContentWrapper>
              }
              />
            </Switch>

            <LoggerWrapper/>

          </div>
        </div>


        <div id={tempPrintDivId} className="print-content-div"></div>

      </div>
    )
  }
}

//withRouter from https://github.com/ReactTraining/react-router/issues/4671
export default withRouter(connect(mapStateToProps, mapDispatchToProps)(App))
